/* OS/2 REXX: identify all *.HLP files setting a .TYPE attribute, */
/* old .TYPE values can be kept (interactive), for unknown *.HLP  */
/* files you can select "text/plain", "Binary Data", or skip it - */
/* human readable "text/plain" is _not_ necessarily unstructured. */

/* Ignore NOTREADY errors, read access failed, nothing happens... */
/* If write access (to modify EAs) fails, you can try once again: */
/* probably you were just looking at the file in another session. */

/* XEAT() error 1 means "almost okay" (one trailing NUL ignored). */
/* XEAT() error 13 is serious (unexpected structure of EA .TYPE). */
/* Other XEAT errors are just the result of SysGetEA or SysPutEA. */

/* Press X, ESC, F3, or Alt-F4 (PM: CANCEL) to abort this script  */
/* after errors (or in any other dialogue resp. PM RxMessageBox). */
/* Press Y, ENTER, or N (PM: YES or NO) to continue after errors. */

/* For details see "select when ..." in procedure DOIT, maybe add */
/* a type for Norton Guide, if you have an application for it, or */
/* replace types "QuickHelp" / "Win HLP file" / "OS/2 HLP file" - */
/* these are my own inventions (with corresponding applications,  */
/* e.g. ViewHelp and/or NewView associated with "OS/2 HLP file"). */

/* BTW, I found an OS/S QuickHelp in the DDK (device driver kit). */
/* I am unsure about *.HLP files starting with magic header 'IS': */
/* maybe replace "Binary Data" by a .TYPE associated with XVIEW ? */

   signal on  novalue  name TRAP ;  signal on  syntax name TRAP
   signal on  failure  name TRAP ;  signal on  halt   name TRAP
   signal on  notready name TRAP ;  signal on  error  name TRAP
   ISVIO = ( PROC() < 3 )        ;  EXPO = 'ISVIO'
   call UTIL 'SysFileTree'       ;  rc = CTOZ()
   if ISVIO then say /* CrLf */  ;  exit rc

CTOZ: procedure expose (EXPO)    /* call SUBS on all local drives */
   call UTIL 'SysDriveMap'       ;  call UTIL 'SysFileSystemType'
   D = SysDriveMap( /**/, 'Local' )

   do N = 1 to words( D )        /* no EA access on CDFS (why ?)  */
      if SysFileSystemType( word( D, N )) = 'CDFS' then  iterate N
      if SUBS(              word( D, N ))          then  return 1
   end N
   return 0

SUBS: procedure expose (EXPO)
   FAIL = 0 ;  SUBS = arg( 1 ) || '\*'
   if sign( SysFileTree( SUBS, 'SUB', 'DO' )) then drop SUB.
   do N = 1 to SUB.0 ;  FAIL = FAIL | SUBS( SUB.N )   ;  end N

   SUBS = SUBS || '.hlp'
   if sign( SysFileTree( SUBS, 'SUB', 'FO' )) then drop SUB.
   do N = 1 to SUB.0 ;  FAIL = FAIL | DOIT( SUB.N )   ;  end N

   return 0                      /* ignore FAIL: CTOZ() continues */

DOIT: procedure expose (EXPO)    /* doing something with one file */
   if ISVIO then call charout /**/, x2c( 0D ) || left( arg(1), 79 )

   signal on notready  name DOIT.LOCK
   TYPE = charin( arg( 1 ), /**/, min( 2, chars( arg( 1 ))))
   call charout arg( 1 ) /* close */   ;  NONE = 'Reserved'
   TEXT = 'text/plain'                 ;  DATA = 'Binary Data'
   rc = XEAT( arg( 1 ), 'OLD', 0 )     ;  CRLF = x2c( 0D0A )
   if rc <> 0  then  return OKAY( CRLF || 'get EA rc =' rc arg( 1 ))

   select
      when TYPE == 'HS' then TYPE = 'OS/2 HLP file'
      when TYPE == '?_' then TYPE = 'Win HLP file'
      when TYPE == 'LN' then TYPE = 'QuickHelp'
      when TYPE == '  ' then TYPE = TEXT           /* RXHELP    ? */
      when TYPE == 'Co' then TYPE = TEXT           /* Copyright ? */
      when TYPE == 'NG' then TYPE = DATA           /* Norton NG ? */
      when TYPE == 'IS' then TYPE = DATA           /* Win XVIEW ? */
      when TYPE == 'Fx' then TYPE = DATA           /* CPI stuff ? */
      when TYPE == 'FB' then TYPE = DATA           /* THelp stuff */
      when TYPE == 'PN' then TYPE = DATA           /* DOS 7 stuff */
      when TYPE == '@ ' then TYPE = TEXT           /* DOS 5 stuff */
      when TYPE == CRLF then TYPE = TEXT           /* wild guess  */
      otherwise
         select                  /* accept single non-trivial EA: */
            when OLD.0 <> 1      then  nop         /* ask user... */
            when OLD.1 == NONE   then  nop         /* ask user... */
            otherwise                  return 0    /* keep old EA */
         end
         select
            when OKAY( CRLF || arg( 1 ) '=>' TEXT ) then TYPE = TEXT
            when OKAY(         arg( 1 ) '=>' DATA ) then TYPE = DATA
            otherwise   return 0 /* maybe user sets a .TYPE later */
         end
         CRLF = ''               /* one CRLF was OKAY() for ISVIO */
   end
   if OLD.0 = 1 then if OLD.1 == TYPE & OLD.. == '' then return 0

   NEW.0 = 1   ;  NEW.1 = TYPE   ;  NEW.. = OLD..
   do O = 1 to OLD.0             /* NEW.. could save old codepage */
      do N = 1 to NEW.0          /* that's rubbish if OLD deleted */
         if OLD.O == NEW.N then  iterate O
      end N
      if OLD.O == NONE     then  iterate O
      if OKAY( CRLF || 'keep' OLD.O 'for' arg( 1 ) '?' ) then do
         NEW.0 = N   ;  NEW.N = OLD.O  ;  CRLF = ''
      end                        /* one CRLF was OKAY() for ISVIO */
   end O

   rc = XEAT( arg( 1 ), 'NEW', 1 )
   if rc = 32  then  do          /* maybe user just checks file ? */
      if OKAY( CRLF || arg( 1 ) 'locked, try again' ) = 0
         then        return 1    /* reported locked file, 1 = ERR */
      rc = XEAT( arg( 1 ), 'NEW', 1 )
      CRLF = ''                  /* one CRLF was OKAY() for ISVIO */
   end
   if rc =  0  then  return 0    /* SUBS() wants boolean, 0 = ok. */
               else  return OKAY( CRLF || 'put EA rc =' rc arg( 1 ))
DOIT.LOCK:           return OKAY( CRLF || 'NOTREADY:' arg( 1 ))

/* see <URL:http://purl.net/xyzzy/rexxtrap.htm>, (c) F. Ellermann */

XEAT:                            /* get or put EA TYPE using STEM */
   /* 1st arg. = FILE, 3rd arg. = 0 to SysGetEA '.TYPE' into STEM */
   /* 2nd arg. = STEM, 3rd arg. = 1 to SysPutEA '.TYPE' from STEM */
   /* STEM.0   = last, STEM.n = n-th EAT_ASCII (0 < n <= STEM.0)  */
   /* STEM..   = '' (one ASCII), or 0 resp. codepage (for MULTI)  */
   if arg( 2, 'o' )  then return abs( /* missing STEM argument */ )
   if arg() <> 3     then return abs( /* missing or extra arg. */ )
   THIS... = arg( 2 )            /* destroying any global THIS... */
   if right( THIS... , 1 ) <> '.' then THIS... = THIS... || '.'
   if arg( 3 )       then return XEAT.PUTEA( arg( 1 ), '.TYPE' )
                     else return XEAT.GETEA( arg( 1 ), '.TYPE' )
XEAT.GETEA: procedure expose ( THIS... )
   call UTIL 'SysGetEA'          ;  MULTI = reverse( x2c( 'FFDF' ))
   ERROR = 13  ;  P = '.'        ;  ASCII = reverse( x2c( 'FFFD' ))
   call value THIS... || 0, 0    /* STEM.0 = 0 if no .TYPE found  */
   call value THIS... || P, ''   /* STEM.. = codepage if MULTI EA */
   call SysGetEA arg( 1 ), arg( 2 ), 'VAL'
   if result <> 0             then return result
   if length( VAL ) = 0       then return 0
   if length( VAL ) < 4       then return ERROR
   parse var VAL EAT 3 SIZE 5 .
   select                        /* handle ASCII as last MULTI EA */
      when EAT == ASCII       then LAST = 1
      when EAT <> MULTI       then return ERROR
      when length( VAL ) < 6  then return ERROR
      otherwise
         call value THIS... || P, c2d( reverse( SIZE ))
         parse var VAL . 5 LAST 7 VAL
         LAST = c2d( reverse( LAST ))
   end
   do N = 1 to LAST
      if length( VAL ) < 4    then return ERROR
      parse var VAL EAT 3 SIZE 5 VAL
      SIZE = c2d( reverse( SIZE ))
      if EAT <> ASCII         then return ERROR
      if length( VAL ) < SIZE then return ERROR
      call value THIS... || 0, N
      call value THIS... || N, left( VAL, SIZE )
      VAL = substr( VAL, SIZE + 1 )
   end N
   if VAL == ''               then return 0        /* really okay */
   if VAL == d2c( 0 )         then return 1        /* almost okay */
                              else return ERROR    /* extra bytes */

XEAT.PUTEA: procedure expose ( THIS... )
   call UTIL 'SysPutEA'          ;  MULTI = reverse( x2c( 'FFDF' ))
   ERROR = 13  ;  P = '.'        ;  ASCII = reverse( x2c( 'FFFD' ))
   LAST = value( THIS... || 0 )  ;  VAL  = ''   ;  N = 0
   if symbol(    THIS... || P ) =  'VAR'      then do
      N = value( THIS... || P )  ;  if N = '' then N = 0
   end                           /* treat undefined codepage as 0 */

   if datatype( LAST, 'w' ) = 0  then return ERROR
   if 10000 <=  LAST | LAST < 0  then return ERROR
   if datatype( N   , 'w' ) = 0  then return ERROR
   if 10000 <=  N    | N    < 0  then return ERROR

   if LAST > 1 | N <> 0 then do  /* use MULTI only if neccessary: */
      N    = reverse( x2c( d2x( N   , 4 )))
      SIZE = reverse( x2c( d2x( LAST, 4 )))
      VAL  = MULTI || N || SIZE  /* x2c(d2x(X, 2*L)) <> d2c(X, L) */
   end
   do N = 1 to LAST              /* WPS accects / keeps void, but */
      EAT  = THIS... || N        /* NOVALUE is definitely illegal */
      if symbol( EAT ) <> 'VAR'  then return ERROR
      EAT  = value( EAT )
      SIZE = reverse( x2c( d2x( length( EAT ), 4 )))
      VAL  = VAL || ASCII || SIZE || EAT
   end N                         /* empty VAL clears the EA .TYPE */
   return SysPutEA( arg( 1 ), arg( 2 ), VAL )

OKAY: procedure                  /* get YES / NO / CANCEL answer: */
   KEY = PROC()                  ;  OUT = 'STDERR'
   YES = x2c( 0D ) || 'Y+1'      ;  NAY = x2c( 7F ) || 'N-0'
   AXE = x2c( 1B ) || 'X=k'      /* F3 (003D) = / Alt-F4 (006B) k */

   select
      when KEY = 1 then do       /* 1 (real) obsolete: here DOS   */
         parse version KEY . .   /* REXX/Personal has no STDERR:  */
         if KEY <> 'REXXSAA' then OUT = '\dev\con'
      end
      when KEY = 3 then do       /* 3 (PM) RxMessageBox() output  */
         parse source KEY  ;  KEY = centre( KEY, 100 )   /* HACK  */
         KEY = RxMessageBox( arg( 1 ), KEY, 'YESNOCANCEL', 'QUERY' )
         if KEY = 6 | KEY = 7 then return 7 - KEY  ;  else exit KEY
      end
      when KEY < 4 then do       /* 0 (fullscreen) and 2 (window) */
         call UTIL 'SysGetKey'   ;  call UTIL 'SysGetMessage'
         YES = YES || word( SysGetMessage( 0 ), 1 )   /* national */
         NAY = NAY || word( SysGetMessage( 0 ), 2 )   /* national */
         AXE = AXE || word( SysGetMessage( 0 ), 3 )   /* national */
      end
      otherwise nop              /* 4 (detached) tested in AKEY() */
   end
   call charout OUT, arg( 1 ) || ': '

   do until sign( pos( KEY, 'YNX' ))
      call charout OUT, x2c( 7 ) ;  KEY = translate( AKEY())
      if pos( KEY, AXE ) > 0 then KEY = 'X'
      if pos( KEY, NAY ) > 0 then KEY = 'N'
      if pos( KEY, YES ) > 0 then KEY = 'Y'
   end

   call lineout OUT, KEY         ;  if KEY = 'Y' then return 1
   if KEY = 'N' then return 0    ;  exit 2

AKEY: procedure                  /* keyboard char. input function */
   KEY = PROC()
   if KEY == 4          then return x2c( 1B )      /* 4: detached */
   if KEY <> 1          then return SysGetKey( 'NoEcho' )
   parse version KEY . .                           /* 1: DOS REXX */
   if KEY == 'REXXSAA'  then return  RxGetKey( 'NoEcho' )
                        else return right( INKEY(), 1 )

PROC: procedure                  /* avoid "unknown function" TRAP */
   parse source OS .             /* for REXXSAA portability abuse */
   if OS <> 'OS/2' then return 1 /* the now obsolete 1: real mode */

   OS = 'ProcessType'            /* assume Sys... = RxProcessType */
   if RxFuncQuery( 'Sys' || OS ) = 0 then return SysProcessType()
   if RxFuncAdd(   'Sys' || OS, 'RxUtils', 'Rx' || OS ) = 0 then do
      signal on syntax name PROC.TRAP  ;  return SysProcessType()
   end                           /* tries RxUtils only once, else */
PROC.TRAP:                       /* force RexxUtil SysProcessType */
   call  RxFuncDrop 'SysProcessType'   ;  signal on syntax name TRAP
   call        UTIL 'SysProcessType'   ;  return SysProcessType()

UTIL: procedure                  /* load necessary RexxUtil entry */
   if RxFuncQuery(  arg( 1 )) then
      if RxFuncAdd( arg( 1 ), 'RexxUtil', arg( 1 )) then
         exit TRAP( "can't add RexxUtil"  arg( 1 ))
   return 0

TRAP:                            /* select REXX exception handler */
   call trace 'O' ;  trace N           /* don't trace interactive */
   parse source TRAP                   /* source on separate line */
   TRAP = x2c( 0D ) || right( '+++', 10 ) TRAP || x2c( 0D0A )
   TRAP = TRAP || right( '+++', 10 )   /* = standard trace prefix */
   TRAP = TRAP strip( condition( 'c' ) 'trap:' condition( 'd' ))
   select
      when wordpos( condition( 'c' ), 'ERROR FAILURE' ) > 0 then do
         if condition( 'd' ) > ''      /* need an additional line */
            then TRAP = TRAP || x2c( 0D0A ) || right( '+++', 10 )
         TRAP = TRAP '(RC' rc || ')'   /* any system error codes  */
         if condition( 'c' ) = 'FAILURE' then rc = -3
      end
      when wordpos( condition( 'c' ), 'HALT SYNTAX'   ) > 0 then do
         if condition( 'c' ) = 'HALT' then rc = 4
         if condition( 'd' ) > '' & condition( 'd' ) <> rc then do
            if condition( 'd' ) <> errortext( rc ) then do
               TRAP = TRAP || x2c( 0D0A ) || right( '+++', 10 )
               TRAP = TRAP errortext( rc )
            end                        /* future condition( 'd' ) */
         end                           /* may use errortext( rc ) */
         else  TRAP = TRAP errortext( rc )
         rc = -rc                      /* rc < 0: REXX error code */
      end
      when condition( 'c' ) = 'NOVALUE'  then rc = -2 /* dubious  */
      when condition( 'c' ) = 'NOTREADY' then rc = -1 /* dubious  */
      otherwise                        /* force non-zero whole rc */
         if datatype( value( 'RC' ), 'W' ) = 0 then rc = 1
         if rc = 0                             then rc = 1
         if condition() = '' then TRAP = TRAP arg( 1 )
   end                                 /* direct: TRAP( message ) */

   TRAP = TRAP || x2c( 0D0A ) || format( sigl, 6 )
   signal on syntax name TRAP.SIGL     /* throw syntax error 3... */
   if 0 < sigl & sigl <= sourceline()  /* if no handle for source */
      then TRAP = TRAP '*-*' strip( sourceline( sigl ))
      else TRAP = TRAP '+++ (source line unavailable)'
TRAP.SIGL:                             /* ...catch syntax error 3 */
   if abbrev( right( TRAP, 2 + 6 ), x2c( 0D0A )) then do
      TRAP = TRAP '+++ (source line unreadable)'   ;  rc = -rc
   end
   select
      when 0 then do                   /* in pipes STDERR: output */
         parse version TRAP.REXX . .   /* REXX/Personal: \dev\con */
         signal on syntax name TRAP.FAIL
         if TRAP.REXX = 'REXXSAA'      /* fails if no more handle */
            then call lineout 'STDERR'  , TRAP
            else call lineout '\dev\con', TRAP
      end
      when 0 then do                   /* OS/2 PM: RxMessageBox() */
         signal on syntax name TRAP.FAIL
         call RxMessageBox ,           /* fails if not in PMREXX  */
            translate( TRAP, ' ', x2c( 0D )), , 'CANCEL', 'WARNING'
      end                              /* replace any CR by blank */
      otherwise   say TRAP ; trace ?L  /* interactive Label trace */
   end

   if condition() = 'SIGNAL' then signal TRAP.EXIT
TRAP.CALL:  return rc                  /* continue after CALL ON  */
TRAP.FAIL:  say TRAP ;  rc = 0 - rc    /* force TRAP error output */
TRAP.EXIT:  exit   rc                  /* exit for any SIGNAL ON  */
