/* OS/2 REXX: check .LONGNAME attributes removing redundant EAs.  */
/* observations:                                                  */
/* - OS/2 translates "invalid" char.s like CR, LF, or "/" to "!"  */
/* - CR LF in .LONGNAME ("!!" in name) could be replaced by " ^"  */
/* - .LONGNAME is a normal length preceeded string (not ASCIIZ)   */
/* - .LONGNAME can be invalid (user can remove invalid .LONGNAME) */
/* - redundant .LONGNAME is silently removed (any error is shown) */
/* - .LONGNAME of ?:\DELETE\* is silently ignored (used by UNDEL) */
/* - .LONGNAME on a CDFS not supported (strange) and not checked  */
/* - .LONGNAME of locked files cannot be read resp. removed       */
/* - if any .LONGNAME is found the user can...                    */
/*   ...accept it (press ENTER or Y)                              */
/*   ...reject it (press N), and if rejected the user can...      */
/*   ...delete it (press ENTER or Y, any error is shown)          */
/*   ...otherwise the SETTINGS view of the file object is opened  */
/* - user can press ESC in all dialogues to terminate the script  */
/* - script can be used on CMD line, via PMREXX, or as PM program */
/* - some directory trees (see below SKIP.0 etc.) can be skipped: */

   SKIP.0 = 4                    /* configure skipped directories */
   SKIP.1 = 'E:\INSTALLED\WPTOOLS'
   SKIP.2 = 'F:\BINB\AWK\GAWKTEST'
   SKIP.3 = 'G:\OS2\INSTALL\JAVA 1.1'
   SKIP.4 = 'G:\OS2\ARCHIVES'

/* -------------------------------------------------------------- */

   signal on  novalue  name TRAP    ;  signal on  syntax name TRAP
   signal on  failure  name TRAP    ;  signal on  halt   name TRAP
   call UTIL 'SysFileTree'          ;  call UTIL 'SysOpenObject'
   call UTIL 'SysGetEA'             ;  call UTIL 'SysPutEA'
   ISVIO  = ( PROC() < 3 )          ;  TOTAL. = 0
   EXPO = 'ISVIO TOTAL. SKIP.'   /* globally EXPOsed variables    */

   THIS = strip( strip( strip( arg( 1 )), /**/, '"' ))
   select
      when THIS = ''   & ISVIO = 0  then  call CTOZ
      when THIS = '-?' | THIS = '?' then  exit HELP()
      when THIS = '-h' | THIS = ''  then  exit HELP()
      when THIS = '-:'              then  call CTOZ
      when THIS <> '*'              then  call SUBS THIS
      otherwise
         THIS = MAKE( 'Check .LONGNAMEs', '%*' )
         exit OKAY( 'WPS object' THIS '(re)created' )
   end

   THIS = TOTAL.0 '.LONGNAMEs (' TOTAL.1 'bytes),' TOTAL.2 'deleted'
   if ISVIO then THIS = x2c( 0D ) || left( THIS, 77 )
   exit OKAY( THIS )

HELP: procedure expose (EXPO)
   parse source . . THIS
   THIS = 'usage:' THIS '[-:|X:|DIR|*]'
   THIS = THIS || x2c( 0D0A ) || '-:    check all  drives'
   THIS = THIS || x2c( 0D0A ) || 'X:    check X:\* drive'
   THIS = THIS || x2c( 0D0A ) || 'DIR   check DIR\* tree'
   if ISVIO then do
      THIS = THIS || x2c( 0D0A ) || '*     (re)create WPS object'
   end
   THIS = THIS || x2c( 0D0A ) || '      press any key'
   return OKAY( THIS ) | 1

CTOZ: procedure expose (EXPO)    /* call SUBS on all local drives */
   call UTIL 'SysDriveMap'          ;  call UTIL 'SysFileSystemType'
   D = SysDriveMap( /**/, 'Local' )

   do N = 1 to words( D )        /* no EA access on CDFS (why ?)  */
      if SysFileSystemType( word( D, N )) = 'CDFS' then  iterate N
      if SUBS(              word( D, N ))          then  return 1
   end N
   return 0

SUBS: procedure expose (EXPO)
   do N = 1 to SKIP.0            /* skip selected directory trees */
      if translate( arg( 1 )) = SKIP.N then return 0
   end N

   FAIL = 0 ;  SUBS = arg( 1 ) || '\*'
   if sign( SysFileTree( SUBS, 'SUB', 'DO' )) then drop SUB.
   do N = 1 to SUB.0 ;  FAIL = FAIL | SUBS( SUB.N )   ;  end N
   do N = 1 to SUB.0 ;  FAIL = FAIL | DOIT( SUB.N )   ;  end N

   if sign( SysFileTree( SUBS, 'SUB', 'FO' )) then drop SUB.
   do N = 1 to SUB.0 ;  FAIL = FAIL | DOIT( SUB.N )   ;  end N

   return FAIL                   /* 1: errors are simply returned */

DOIT: procedure expose (EXPO)    /* 1: error for e.g. EA DATA. SF */
   if ISVIO then call charout /**/, x2c( 0D ) || left( arg(1), 79 )

   L = max( lastpos( '\', arg( 1 )), lastpos( '/', arg( 1 )))
   N = substr( arg( 1 ), L + 1 ) /* hardwired: ignore ?:\DELETE\* */
   if substr( arg( 1 ), 2 ) == ':\DELETE\' || N then  return 0

   R = SysGetEA( arg( 1 ), '.LONGNAME', 'L' )
   if R == 32  then  return 1 - DONE( arg( 1 ), 'locked'  )
   if R <> 0   then  return 1 - DONE( arg( 1 ), 'error' R )
   if L == ''  then  return R    /* no .LONGNAME, return no error */
   TOTAl.0 = TOTAL.0 + 1
   TOTAl.1 = TOTAL.1 + length( L )
   parse var L T =3 L =5 NAME ;  L = c2d( reverse( L ))
   SHOW = translate( NAME, /**/, xrange( d2c( 0 ), d2c( 31 )), '?' )

   if reverse( T ) <> d2c( -3, 2 ) | L <> length( NAME ) then do
      R = 'unexpected type' c2x( reverse( T )) 'length' L 'for'
      R = R || x2c( 0D0A ) || SHOW
      R = R || x2c( 0D0A ) || 'delete invalid .LONGNAME for' N
      if DONE( arg( 1 ), R )        then return KILL( arg( 1 ))
                                    else return 0
   end

   T = strip( NAME, 'Trailing', d2c( 0 ))
   if T == N | T == translate( N )  then return KILL( arg( 1 ))
   T = translate( T, '!!!!', '/\' || x2c( 0D0A ))
   if T == N | T == translate( N )  then return 0

   T = '.LONGNAME for' N   ;  R = SHOW T 'ok.'
   if 76 < length( R )  then  R = SHOW || x2c( 0D0A ) || T 'ok.'
   if DONE( arg( 1 ), R )           then return 0
   if OKAY( 'delete' T )            then return KILL( arg( 1 ))
   if SysOpenObject( arg( 1 ), 'SETTINGS', 1 ) then /**/ , /**/
   if SysOpenObject( arg( 1 ), 'SETTINGS', 1 ) then   return 0
   return 1 - OKAY( 'cannot open SETTINGS for' N )

KILL:  procedure expose (EXPO)   /* try to remove a .LONGNAME EA  */
   R = SysPutEA( arg( 1 ), '.LONGNAME', '' )
   if R = 0 then  TOTAL.2 = TOTAL.2 +1 ;  else  do
      if R <> 32  then  R = 'error' R  ;  else  R = 'locked'
      R = 1 - DONE( arg( 1 ), R )
   end
   return R

DONE:  procedure expose (EXPO)   /* 1st line: path, 2nd line: msg */
   LINE = x2c( 0D ) || left( arg( 1 ), 79 ) || x2c( 0D0A )
   return OKAY( LINE || arg( 2 ))

/* see <URL:http://purl.net/xyzzy/rexxtrap.htm>, (c) F. Ellermann */

OKAY: procedure                  /* get YES / NO / CANCEL answer: */
   KEY = PROC()                  ;  OUT = 'STDERR'
   YES = x2c( 0D ) || 'Y+1'      ;  NAY = x2c( 7F ) || 'N-0'
   AXE = x2c( 1B ) || 'X=k'      /* F3 (003D) = / Alt-F4 (006B) k */

   select
      when KEY = 1 then do       /* 1 (real) obsolete: here DOS   */
         parse version KEY . .   /* REXX/Personal has no STDERR:  */
         if KEY <> 'REXXSAA' then OUT = '\dev\con'
      end
      when KEY = 3 then do       /* 3 (PM) RxMessageBox() output  */
         parse source KEY  ;  KEY = centre( KEY, 100 )   /* HACK  */
         KEY = RxMessageBox( arg( 1 ), KEY, 'YESNOCANCEL', 'QUERY' )
         if KEY = 6 | KEY = 7 then return 7 - KEY  ;  else exit KEY
      end
      when KEY < 4 then do       /* 0 (fullscreen) and 2 (window) */
         call UTIL 'SysGetKey'   ;  call UTIL 'SysGetMessage'
         YES = YES || word( SysGetMessage( 0 ), 1 )   /* national */
         NAY = NAY || word( SysGetMessage( 0 ), 2 )   /* national */
         AXE = AXE || word( SysGetMessage( 0 ), 3 )   /* national */
      end
      otherwise nop              /* 4 (detached) tested in AKEY() */
   end
   call charout OUT, arg( 1 ) || ': '

   do until sign( pos( KEY, 'YNX' ))
      call charout OUT, x2c( 7 ) ;  KEY = translate( AKEY())
      if pos( KEY, AXE ) > 0 then KEY = 'X'
      if pos( KEY, NAY ) > 0 then KEY = 'N'
      if pos( KEY, YES ) > 0 then KEY = 'Y'
   end

   call lineout OUT, KEY         ;  if KEY = 'Y' then return 1
   if KEY = 'N' then return 0    ;  exit 2

MAKE: procedure                  /* recreate or update WPS object */
   /* 1st arg: optional object title, default name of source      */
   /* 2nd arg: optional start arg.s,  use '[txt]' if interactive  */
   /* 3rd arg: optional start directory, default TMP environment  */
   TMP = value( 'TMP',, 'OS2ENVIRONMENT' )
   if TMP = '' then TMP = directory()
   call UTIL 'SysCreateObject'      ;  call UTIL 'SysGetEA'
   parse upper source . . SRC       ;  POS = lastpos( '\', SRC )
   TXT = substr( SRC, POS + 1 )     ;  DIR = left( SRC, POS )
   OBJ = '<' || TXT || '>'          ;  POS = lastpos( '.', TXT )
   TXT = left( TXT, POS - 1 )       ;  NEW = 0

   ICO = stream( DIR || TXT || '.ICO', 'c', 'query exists' )
   if ICO = '' & SysGetEA( SRC, '.ICON', 'POS' ) = 0 then do
      call UTIL 'SysTempFileName'   ;  call UTIL 'SysFileDelete'
      signal on notready name TRAP  ;  SET = substr( POS, 5 )
      NEW = ( length( SET ) = c2d( reverse( substr( POS, 3, 2 ))))
      NEW = NEW & abbrev( POS, x2c( 'F9FF' ))
      if NEW then do             /* SysTempFileName error ignored */
         ICO = SysTempFileName( TMP || '\TMP?????.ICO' )
         call charout ICO, SET      ;  call charout ICO
      end
   end

   if arg( 3, 'O' ) then DIR = TMP  ;  else DIR = arg( 3 )
   if arg( 1, 'E' ) then TXT = arg( 1 )

   SET = 'EXENAME=*;PARAMETERS=/C' strip( SRC arg( 2 ))
   SET = SET || ';MINIMIZED=YES;PROGTYPE=PM'
   if ICO <> '' then SET = SET || ';ICONFILE=' || ICO
   SET = SET || ';STARTUPDIR=' || DIR || ';OBJECTID=' || OBJ || ';'

   POS = '<WP_DESKTOP>'
   POS = SysCreateObject( 'WPProgram', TXT, POS, SET, 'Update' )
   if NEW then call SysFileDelete ICO
   if POS then return OBJ        /* ready to create shadow etc.   */
          else exit TRAP( 'fatal - cannot update' OBJ )

AKEY: procedure                  /* keyboard char. input function */
   KEY = PROC()
   if KEY == 4          then return x2c( 1B )      /* 4: detached */
   if KEY <> 1          then return SysGetKey( 'NoEcho' )
   parse version KEY . .                           /* 1: DOS REXX */
   if KEY == 'REXXSAA'  then return  RxGetKey( 'NoEcho' )
                        else return right( INKEY(), 1 )

PROC: procedure                  /* avoid "unknown function" TRAP */
   parse source OS .             /* for REXXSAA portability abuse */
   if OS <> 'OS/2' then return 1 /* the now obsolete 1: real mode */

   OS = 'ProcessType'            /* assume Sys... = RxProcessType */
   if RxFuncQuery( 'Sys' || OS ) = 0 then return SysProcessType()
   if RxFuncAdd(   'Sys' || OS, 'RxUtils', 'Rx' || OS ) = 0 then do
      signal on syntax name PROC.TRAP  ;  return SysProcessType()
   end                           /* tries RxUtils only once, else */
PROC.TRAP:                       /* force RexxUtil SysProcessType */
   call  RxFuncDrop 'SysProcessType'   ;  signal on syntax name TRAP
   call        UTIL 'SysProcessType'   ;  return SysProcessType()

UTIL: procedure                  /* load necessary RexxUtil entry */
   if RxFuncQuery(  arg( 1 )) then
      if RxFuncAdd( arg( 1 ), 'RexxUtil', arg( 1 )) then
         exit TRAP( "can't add RexxUtil"  arg( 1 ))
   return 0

TRAP:                            /* select REXX exception handler */
   call trace 'O' ;  trace N           /* don't trace interactive */
   parse source TRAP                   /* source on separate line */
   TRAP = x2c( 0D ) || right( '+++', 10 ) TRAP || x2c( 0D0A )
   TRAP = TRAP || right( '+++', 10 )   /* = standard trace prefix */
   TRAP = TRAP strip( condition( 'c' ) 'trap:' condition( 'd' ))
   select
      when wordpos( condition( 'c' ), 'ERROR FAILURE' ) > 0 then do
         if condition( 'd' ) > ''      /* need an additional line */
            then TRAP = TRAP || x2c( 0D0A ) || right( '+++', 10 )
         TRAP = TRAP '(RC' rc || ')'   /* any system error codes  */
         if condition( 'c' ) = 'FAILURE' then rc = -3
      end
      when wordpos( condition( 'c' ), 'HALT SYNTAX'   ) > 0 then do
         if condition( 'c' ) = 'HALT' then rc = 4
         if condition( 'd' ) > '' & condition( 'd' ) <> rc then do
            if condition( 'd' ) <> errortext( rc ) then do
               TRAP = TRAP || x2c( 0D0A ) || right( '+++', 10 )
               TRAP = TRAP errortext( rc )
            end                        /* future condition( 'd' ) */
         end                           /* may use errortext( rc ) */
         else  TRAP = TRAP errortext( rc )
         rc = -rc                      /* rc < 0: REXX error code */
      end
      when condition( 'c' ) = 'NOVALUE'  then rc = -2 /* dubious  */
      when condition( 'c' ) = 'NOTREADY' then rc = -1 /* dubious  */
      otherwise                        /* force non-zero whole rc */
         if datatype( value( 'RC' ), 'W' ) = 0 then rc = 1
         if rc = 0                             then rc = 1
         if condition() = '' then TRAP = TRAP arg( 1 )
   end                                 /* direct: TRAP( message ) */

   TRAP = TRAP || x2c( 0D0A ) || format( sigl, 6 )
   signal on syntax name TRAP.SIGL     /* throw syntax error 3... */
   if 0 < sigl & sigl <= sourceline()  /* if no handle for source */
      then TRAP = TRAP '*-*' strip( sourceline( sigl ))
      else TRAP = TRAP '+++ (source line unavailable)'
TRAP.SIGL:                             /* ...catch syntax error 3 */
   if abbrev( right( TRAP, 2 + 6 ), x2c( 0D0A )) then do
      TRAP = TRAP '+++ (source line unreadable)'   ;  rc = -rc
   end
   select
      when 0 then do                   /* in pipes STDERR: output */
         parse version TRAP.REXX . .   /* REXX/Personal: \dev\con */
         signal on syntax name TRAP.FAIL
         if TRAP.REXX = 'REXXSAA'      /* fails if no more handle */
            then call lineout 'STDERR'  , TRAP
            else call lineout '\dev\con', TRAP
      end
      when 1 then do                   /* OS/2 PM: RxMessageBox() */
         signal on syntax name TRAP.FAIL
         call RxMessageBox ,           /* fails if not in PMREXX  */
            translate( TRAP, ' ', x2c( 0D )), , 'CANCEL', 'WARNING'
      end                              /* replace any CR by blank */
      otherwise   say TRAP ; trace ?L  /* interactive Label trace */
   end

   if condition() = 'SIGNAL' then signal TRAP.EXIT
TRAP.CALL:  return rc                  /* continue after CALL ON  */
TRAP.FAIL:  say TRAP ;  rc = 0 - rc    /* force TRAP error output */
TRAP.EXIT:  exit   rc                  /* exit for any SIGNAL ON  */
