/* OS/2 REXX example: recurse subdirectories doing nothing... ;-) */
/* Or rather show all locked ("busy") files in a directory tree   */
/* ignoring obvious cases of hidden / system / read only files.   */

/* Caveat:  SysFileTree() blocks for "illegal" files, i.e. names  */
/* with illegal characters also not handled by a simple DIR like  */
/* x2c( E4 F6 FC ), the lower case of x2c( C4 D6 DC ) in CP 1004. */

   signal on  novalue  name TRAP ;  signal on  syntax name TRAP
   signal on  failure  name TRAP ;  signal on  halt   name TRAP
   call UTIL 'SysFileTree'

   do setlocal() until endlocal()
      HERE = directory()
      ROOT = directory( strip( strip( strip( arg(1)),, '"' )))
      call directory HERE
      if ROOT = '' then do
         parse source . . HERE   ;  say 'usage:' HERE 'directory'
         if arg( 1 ) <> '' then say 'error:' arg( 1 ) ;  exit 1
      end
      if sign( pos( right( ROOT, 1 ), '/\' ))
         then  rc = SUBS( left( ROOT, length( ROOT ) -1 ))
         else  rc = SUBS( ROOT ) /* get rid of trailing backslash */
   end
   exit rc                       /* novalue if setlocal() failure */

SUBS: procedure
   FAIL = 0 ;  SUBS = arg( 1 ) || '\*'
   if sign( SysFileTree( SUBS, 'SUB', 'DO' )) then drop SUB.
   do N = 1 to SUB.0 ;  FAIL = FAIL | SUBS( SUB.N )   ;  end N

   if sign( SysFileTree( SUBS, 'SUB', 'FO' )) then drop SUB.
   do N = 1 to SUB.0 ;  FAIL = FAIL | DOIT( SUB.N )   ;  end N

   return FAIL

DOIT: procedure                  /* doing something with one file */
   signal on notready  name DOIT.LOCK
   return charout( arg( 1 ))
DOIT.LOCK:
   if sign( SysFileTree( arg( 1 ), 'SUB', 'FT' )) then drop SUB.
   if SUB.0 <> 1 then drop SUB.  /* force novalue if ambiguous    */
   select
      when substr( word( SUB.1, 3 ), 3, 1 ) = 'H'  then  return 0
      when substr( word( SUB.1, 3 ), 4, 1 ) = 'R'  then  return 0
      when substr( word( SUB.1, 3 ), 5, 1 ) = 'S'  then  return 0
      otherwise   say 'locked:' arg( 1 )              ;  return 1
   end

/* see <URL:http://purl.net/xyzzy/rexxtrap.htm>, (c) F. Ellermann */

UTIL: procedure                  /* load necessary RexxUtil entry */
   if RxFuncQuery(  arg( 1 )) then
      if RxFuncAdd( arg( 1 ), 'RexxUtil', arg( 1 )) then
         exit TRAP( "can't add RexxUtil"  arg( 1 ))
   return 0

TRAP:                            /* select REXX exception handler */
   call trace 'O' ;  trace N           /* don't trace interactive */
   parse source TRAP                   /* source on separate line */
   TRAP = x2c( 0D ) || right( '+++', 10 ) TRAP || x2c( 0D0A )
   TRAP = TRAP || right( '+++', 10 )   /* = standard trace prefix */
   TRAP = TRAP strip( condition( 'c' ) 'trap:' condition( 'd' ))
   select
      when wordpos( condition( 'c' ), 'ERROR FAILURE' ) > 0 then do
         if condition( 'd' ) > ''      /* need an additional line */
            then TRAP = TRAP || x2c( 0D0A ) || right( '+++', 10 )
         TRAP = TRAP '(RC' rc || ')'   /* any system error codes  */
         if condition( 'c' ) = 'FAILURE' then rc = -3
      end
      when wordpos( condition( 'c' ), 'HALT SYNTAX'   ) > 0 then do
         if condition( 'c' ) = 'HALT' then rc = 4
         if condition( 'd' ) > '' & condition( 'd' ) <> rc then do
            if condition( 'd' ) <> errortext( rc ) then do
               TRAP = TRAP || x2c( 0D0A ) || right( '+++', 10 )
               TRAP = TRAP errortext( rc )
            end                        /* future condition( 'd' ) */
         end                           /* may use errortext( rc ) */
         else  TRAP = TRAP errortext( rc )
         rc = -rc                      /* rc < 0: REXX error code */
      end
      when condition( 'c' ) = 'NOVALUE'  then rc = -2 /* dubious  */
      when condition( 'c' ) = 'NOTREADY' then rc = -1 /* dubious  */
      otherwise                        /* force non-zero whole rc */
         if datatype( value( 'RC' ), 'W' ) = 0 then rc = 1
         if rc = 0                             then rc = 1
         if condition() = '' then TRAP = TRAP arg( 1 )
   end                                 /* direct: TRAP( message ) */

   TRAP = TRAP || x2c( 0D0A ) || format( sigl, 6 )
   signal on syntax name TRAP.SIGL     /* throw syntax error 3... */
   if 0 < sigl & sigl <= sourceline()  /* if no handle for source */
      then TRAP = TRAP '*-*' strip( sourceline( sigl ))
      else TRAP = TRAP '+++ (source line unavailable)'
TRAP.SIGL:                             /* ...catch syntax error 3 */
   if abbrev( right( TRAP, 2 + 6 ), x2c( 0D0A )) then do
      TRAP = TRAP '+++ (source line unreadable)'   ;  rc = -rc
   end
   select
      when 0 then do                   /* in pipes STDERR: output */
         parse version TRAP.REXX . .   /* REXX/Personal: \dev\con */
         signal on syntax name TRAP.FAIL
         if TRAP.REXX = 'REXXSAA'      /* fails if no more handle */
            then call lineout 'STDERR'  , TRAP
            else call lineout '\dev\con', TRAP
      end
      when 0 then do                   /* OS/2 PM: RxMessageBox() */
         signal on syntax name TRAP.FAIL
         call RxMessageBox ,           /* fails if not in PMREXX  */
            translate( TRAP, ' ', x2c( 0D )), , 'CANCEL', 'WARNING'
      end                              /* replace any CR by blank */
      otherwise   say TRAP ; trace ?L  /* interactive Label trace */
   end

   if condition() = 'SIGNAL' then signal TRAP.EXIT
TRAP.CALL:  return rc                  /* continue after CALL ON  */
TRAP.FAIL:  say TRAP ;  rc = 0 - rc    /* force TRAP error output */
TRAP.EXIT:  exit   rc                  /* exit for any SIGNAL ON  */
