/* OS/2 REXX:  display files with a specified EA on local disks - */
/* maybe use it to delete extended attribute .PREVCLASS on Warp 3 */

   signal on  novalue  name TRAP    ;  signal on  syntax name TRAP
   signal on  failure  name TRAP    ;  signal on  halt   name TRAP
   call UTIL 'SysFileTree'          ;  call UTIL 'SysQueryEAList'
   call UTIL 'SysPutEA'

   CRLF = x2c( 0D0A )               ;  ISVIO = ( PROC() < 3 )
   EXPO = 'ISVIO XARG CRLF'         ;  parse source . . TEXT
   TEXT = ' usage:' TEXT '[EA]'     ;  XARG = arg( 1 )
   TEXT = TEXT CRLF 'displays all files on all local disks with an'
   TEXT = TEXT CRLF 'extended attribute EA allowing to remove it. '
   select
      when XARG = '' then  exit WAIT( TEXT )
      when CTOZ()    then  exit WAIT( 'aborted' )
      otherwise            exit 0
   end

CTOZ: procedure expose (EXPO)    /* call SUBS on all local drives */
   call UTIL 'SysDriveMap'          ;  call UTIL 'SysFileSystemType'
   D = SysDriveMap( /**/, 'Local' )

   do N = 1 to words( D )        /* no EA access on CDFS (why ?)  */
      if SysFileSystemType( word( D, N )) = 'CDFS' then  iterate N
      if SUBS(              word( D, N ))          then  return 1
   end N
   return 0

SUBS: procedure expose (EXPO)
   FAIL = 0 ;  SUBS = arg( 1 ) || '\*'
   if sign( SysFileTree( SUBS, 'SUB', 'DO' )) then drop SUB.
   do N = 1 to SUB.0 ;  FAIL = FAIL | SUBS( SUB.N )   ;  end N
   do N = 1 to SUB.0 ;  FAIL = FAIL | DOIT( SUB.N )   ;  end N

   if sign( SysFileTree( SUBS, 'SUB', 'FO' )) then drop SUB.
   do N = 1 to SUB.0 ;  FAIL = FAIL | DOIT( SUB.N )   ;  end N

   return FAIL                   /* 1: errors are simply ignored  */

DOIT: procedure expose (EXPO)    /* 1: error for e.g. EA DATA. SF */
   if ISVIO then call charout /**/, left( arg(1), 79 ) || x2c( 0D )

   THIS = SysQueryEAList( arg( 1 ), 'EA' )
   if THIS = 32   then  return 1 - WAIT( 'locked:' arg( 1 ))
   if THIS <> 0   then  return 1 - WAIT( 'error' THIS arg( 1 ))

   do N = 1 to EA.0
      if translate( EA.N ) <> translate( XARG ) then iterate N

      THIS = arg( 1 ) '- delete EA "' EA.N || '" ?'
      if length( THIS ) > 80 - 4 * ISVIO
         then THIS = overlay( CRLF, THIS, 1 + length( arg( 1 )))
      if OKAY( THIS ) = 0                       then iterate N

      call SysPutEA arg( 1 ), EA.N, ''
      if result = 0                             then iterate N
      if result = 32 then call WAIT 'locked:' arg( 1 )
                     else call WAIT 'error' result arg( 1 )
      if result = 0  then return 1
   end N

   return 0

/* see <URL:http://purl.net/xyzzy/rexxtrap.htm>, (c) F. Ellermann */

WAIT: procedure                  /* get OKay resp. CANCEL answer: */
   KEY = PROC()                  ;  OUT = 'STDERR'

   select
      when KEY = 1 then do       /* 1 (real) obsolete: here DOS   */
         parse version KEY . .   /* REXX/Personal has no STDERR:  */
         if KEY <> 'REXXSAA' then OUT = '\dev\con'
      end
      when KEY = 3 then do       /* 3 (PM) RxMessageBox() output  */
         parse source KEY  ;  KEY = centre( KEY, 100 )   /* HACK  */
         KEY = RxMessageBox( arg( 1 ), KEY, 'OKCANCEL', 'ASTERISK' )
         return KEY = 1 | KEY = 6 | KEY = 8
      end                        /* 0 (fullscreen) and 2 (window) */
      when KEY < 4 then call UTIL 'SysGetKey'
      otherwise nop              /* 4 (detached) tested in AKEY() */
   end
   call charout OUT, arg( 1 ) || x2c( 7 )

   do until c2d( KEY ) <> 0 & c2d( KEY ) <> 224
      KEY = AKEY()
   end
   call lineout OUT, ''          /* hardwiring F3 '=', Alt-F4 'k' */
   return KEY <> x2c( 1B ) & KEY <> '=' & KEY <> 'k'

OKAY: procedure                  /* get YES / NO / CANCEL answer: */
   KEY = PROC()                  ;  OUT = 'STDERR'
   YES = x2c( 0D ) || 'Y+1'      ;  NAY = x2c( 7F ) || 'N-0'
   AXE = x2c( 1B ) || 'X=k'      /* F3 (003D) = / Alt-F4 (006B) k */

   select
      when KEY = 1 then do       /* 1 (real) obsolete: here DOS   */
         parse version KEY . .   /* REXX/Personal has no STDERR:  */
         if KEY <> 'REXXSAA' then OUT = '\dev\con'
      end
      when KEY = 3 then do       /* 3 (PM) RxMessageBox() output  */
         parse source KEY  ;  KEY = centre( KEY, 100 )   /* HACK  */
         KEY = RxMessageBox( arg( 1 ), KEY, 'YESNOCANCEL', 'QUERY' )
         if KEY = 6 | KEY = 7 then return 7 - KEY  ;  else exit KEY
      end
      when KEY < 4 then do       /* 0 (fullscreen) and 2 (window) */
         call UTIL 'SysGetKey'   ;  call UTIL 'SysGetMessage'
         YES = YES || word( SysGetMessage( 0 ), 1 )   /* national */
         NAY = NAY || word( SysGetMessage( 0 ), 2 )   /* national */
         AXE = AXE || word( SysGetMessage( 0 ), 3 )   /* national */
      end
      otherwise nop              /* 4 (detached) tested in AKEY() */
   end
   call charout OUT, arg( 1 ) || ': '

   do until sign( pos( KEY, 'YNX' ))
      call charout OUT, x2c( 7 ) ;  KEY = translate( AKEY())
      if pos( KEY, AXE ) > 0 then KEY = 'X'
      if pos( KEY, NAY ) > 0 then KEY = 'N'
      if pos( KEY, YES ) > 0 then KEY = 'Y'
   end

   call lineout OUT, KEY         ;  if KEY = 'Y' then return 1
   if KEY = 'N' then return 0    ;  exit 2

AKEY: procedure                  /* keyboard char. input function */
   KEY = PROC()
   if KEY == 4          then return x2c( 1B )      /* 4: detached */
   if KEY <> 1          then return SysGetKey( 'NoEcho' )
   parse version KEY . .                           /* 1: DOS REXX */
   if KEY == 'REXXSAA'  then return  RxGetKey( 'NoEcho' )
                        else return right( INKEY(), 1 )

PROC: procedure                  /* avoid "unknown function" TRAP */
   parse source OS .             /* for REXXSAA portability abuse */
   if OS <> 'OS/2' then return 1 /* the now obsolete 1: real mode */

   OS = 'ProcessType'            /* assume Sys... = RxProcessType */
   if RxFuncQuery( 'Sys' || OS ) = 0 then return SysProcessType()
   if RxFuncAdd(   'Sys' || OS, 'RxUtils', 'Rx' || OS ) = 0 then do
      signal on syntax name PROC.TRAP  ;  return SysProcessType()
   end                           /* tries RxUtils only once, else */
PROC.TRAP:                       /* force RexxUtil SysProcessType */
   call  RxFuncDrop 'SysProcessType'   ;  signal on syntax name TRAP
   call        UTIL 'SysProcessType'   ;  return SysProcessType()

UTIL: procedure                  /* load necessary RexxUtil entry */
   if RxFuncQuery(  arg( 1 )) then
      if RxFuncAdd( arg( 1 ), 'RexxUtil', arg( 1 )) then
         exit TRAP( "can't add RexxUtil"  arg( 1 ))
   return 0

TRAP:                            /* select REXX exception handler */
   call trace 'O' ;  trace N           /* don't trace interactive */
   parse source TRAP                   /* source on separate line */
   TRAP = x2c( 0D ) || right( '+++', 10 ) TRAP || x2c( 0D0A )
   TRAP = TRAP || right( '+++', 10 )   /* = standard trace prefix */
   TRAP = TRAP strip( condition( 'c' ) 'trap:' condition( 'd' ))
   select
      when wordpos( condition( 'c' ), 'ERROR FAILURE' ) > 0 then do
         if condition( 'd' ) > ''      /* need an additional line */
            then TRAP = TRAP || x2c( 0D0A ) || right( '+++', 10 )
         TRAP = TRAP '(RC' rc || ')'   /* any system error codes  */
         if condition( 'c' ) = 'FAILURE' then rc = -3
      end
      when wordpos( condition( 'c' ), 'HALT SYNTAX'   ) > 0 then do
         if condition( 'c' ) = 'HALT' then rc = 4
         if condition( 'd' ) > '' & condition( 'd' ) <> rc then do
            if condition( 'd' ) <> errortext( rc ) then do
               TRAP = TRAP || x2c( 0D0A ) || right( '+++', 10 )
               TRAP = TRAP errortext( rc )
            end                        /* future condition( 'd' ) */
         end                           /* may use errortext( rc ) */
         else  TRAP = TRAP errortext( rc )
         rc = -rc                      /* rc < 0: REXX error code */
      end
      when condition( 'c' ) = 'NOVALUE'  then rc = -2 /* dubious  */
      when condition( 'c' ) = 'NOTREADY' then rc = -1 /* dubious  */
      otherwise                        /* force non-zero whole rc */
         if datatype( value( 'RC' ), 'W' ) = 0 then rc = 1
         if rc = 0                             then rc = 1
         if condition() = '' then TRAP = TRAP arg( 1 )
   end                                 /* direct: TRAP( message ) */

   TRAP = TRAP || x2c( 0D0A ) || format( sigl, 6 )
   signal on syntax name TRAP.SIGL     /* throw syntax error 3... */
   if 0 < sigl & sigl <= sourceline()  /* if no handle for source */
      then TRAP = TRAP '*-*' strip( sourceline( sigl ))
      else TRAP = TRAP '+++ (source line unavailable)'
TRAP.SIGL:                             /* ...catch syntax error 3 */
   if abbrev( right( TRAP, 2 + 6 ), x2c( 0D0A )) then do
      TRAP = TRAP '+++ (source line unreadable)'   ;  rc = -rc
   end
   select
      when 0 then do                   /* in pipes STDERR: output */
         parse version TRAP.REXX . .   /* REXX/Personal: \dev\con */
         signal on syntax name TRAP.FAIL
         if TRAP.REXX = 'REXXSAA'      /* fails if no more handle */
            then call lineout 'STDERR'  , TRAP
            else call lineout '\dev\con', TRAP
      end
      when 0 then do                   /* OS/2 PM: RxMessageBox() */
         signal on syntax name TRAP.FAIL
         call RxMessageBox ,           /* fails if not in PMREXX  */
            translate( TRAP, ' ', x2c( 0D )), , 'CANCEL', 'WARNING'
      end                              /* replace any CR by blank */
      otherwise   say TRAP ; trace ?L  /* interactive Label trace */
   end

   if condition() = 'SIGNAL' then signal TRAP.EXIT
TRAP.CALL:  return rc                  /* continue after CALL ON  */
TRAP.FAIL:  say TRAP ;  rc = 0 - rc    /* force TRAP error output */
TRAP.EXIT:  exit   rc                  /* exit for any SIGNAL ON  */
