/* OS/2 REXX: remove specified EA .TYPE from all files using XEAT */

   signal on  novalue  name TRAP    ;  signal on  syntax name TRAP
   signal on  failure  name TRAP    ;  signal on  halt   name TRAP
   call UTIL 'SysGetMessage'        ;  call UTIL 'SysFileTree'
   parse source . . THIS            ;  EXPO = 'DEAD'

   DEAD = strip( strip( strip( arg( 1 )),, '"' ))
   if DEAD = '' then do
      say 'usage:' THIS '[TYPE]'
      say 'removes TYPE from .TYPE EAs of all files on all disks'
      say 'without asking questions.  Test it with TYPE Reserved'
      say '(case sensitive).  "Reserved" is added by the WPS if'
      say 'you remove the last .TYPE "Plain Text", which doesn''t'
      say 'physically exist ("Plain Text" is actually no .TYPE).'
      exit 1
   end

   if CTOZ() then do
      say   ;  say THIS 'aborted'   ;  exit 1
   end
   say x2c( 0D ) left( '', 77 ) x2c( 0D ) 'All' DEAD 'removed.'
   exit 0

CTOZ: procedure expose (EXPO)    /* call SUBS on all local drives */
   call UTIL 'SysDriveMap'          ;  call UTIL 'SysFileSystemType'
   D = SysDriveMap( /**/, 'Local' )

   do N = 1 to words( D )        /* no EA access on CDFS (why ?)  */
      if SysFileSystemType( word( D, N )) = 'CDFS' then  iterate N
      if SUBS(              word( D, N ))          then  return 1
   end N
   return 0

SUBS: procedure expose (EXPO)
   FAIL = 0 ;  SUBS = arg( 1 ) || '\*'
   if sign( SysFileTree( SUBS, 'SUB', 'DO' )) then drop SUB.
   do N = 1 to SUB.0 ;  FAIL = FAIL | SUBS( SUB.N )   ;  end N
   do N = 1 to SUB.0 ;  FAIL = FAIL | DOIT( SUB.N )   ;  end N

   if sign( SysFileTree( SUBS, 'SUB', 'FO' )) then drop SUB.
   do N = 1 to SUB.0 ;  FAIL = FAIL | DOIT( SUB.N )   ;  end N

   return FAIL                   /* 1: errors are simply ignored  */

DOIT: procedure expose (EXPO)    /* 1: error for e.g. EA DATA. SF */
   call charout /**/, x2c( 0D ) || left( arg(1), 79 )

   X = XEAT( arg( 1 ), 'X', 0 )
   select
      when X   = 32  then return 0
      when X   >  1  then exit TRAP( 'error' X SysGetMessage( X ))
      when X.0 =  0  then return 0
      otherwise nop
   end

   Y.. = X..      ;  Y.0 = 0
   do X = 1 to X.0
      if X.X == DEAD then leave X
      Y.X = X.X   ;  Y.0 = X
   end X
   if X > X.0 then return 0

   do Y = X to X.0 - 1
      X = Y + 1
      Y.Y = X.X   ;  Y.0 = Y
   end Y

   call charout /**/, x2c( 0D ) || arg( 1 )
   X = XEAT( arg( 1 ), 'Y', 1 )

   select
      when X   = 32  then  say ' locked'
      when X   >  1  then  say ' ' || SysGetMessage( X )
      otherwise            say ' patched'
   end
   return 0

/* see <URL:http://purl.net/xyzzy/rexxtrap.htm>, (c) F. Ellermann */

XEAT:                            /* get or put EA TYPE using STEM */
   /* 1st arg. = FILE, 3rd arg. = 0 to SysGetEA '.TYPE' into STEM */
   /* 2nd arg. = STEM, 3rd arg. = 1 to SysPutEA '.TYPE' from STEM */
   /* STEM.0   = last, STEM.n = n-th EAT_ASCII (0 < n <= STEM.0)  */
   /* STEM..   = '' (one ASCII), or 0 resp. codepage (for MULTI)  */
   if arg( 2, 'o' )  then return abs( /* missing STEM argument */ )
   if arg() <> 3     then return abs( /* missing or extra arg. */ )
   THIS... = arg( 2 )            /* destroying any global THIS... */
   if right( THIS... , 1 ) <> '.' then THIS... = THIS... || '.'
   if arg( 3 )       then return XEAT.PUTEA( arg( 1 ), '.TYPE' )
                     else return XEAT.GETEA( arg( 1 ), '.TYPE' )
XEAT.GETEA: procedure expose ( THIS... )
   call UTIL 'SysGetEA'          ;  MULTI = reverse( x2c( 'FFDF' ))
   ERROR = 13  ;  P = '.'        ;  ASCII = reverse( x2c( 'FFFD' ))
   call value THIS... || 0, 0    /* STEM.0 = 0 if no .TYPE found  */
   call value THIS... || P, ''   /* STEM.. = codepage if MULTI EA */
   call SysGetEA arg( 1 ), arg( 2 ), 'VAL'
   if result <> 0             then return result
   if length( VAL ) = 0       then return 0
   if length( VAL ) < 4       then return ERROR
   parse var VAL EAT 3 SIZE 5 .
   select                        /* handle ASCII as last MULTI EA */
      when EAT == ASCII       then LAST = 1
      when EAT <> MULTI       then return ERROR
      when length( VAL ) < 6  then return ERROR
      otherwise
         call value THIS... || P, c2d( reverse( SIZE ))
         parse var VAL . 5 LAST 7 VAL
         LAST = c2d( reverse( LAST ))
   end
   do N = 1 to LAST
      if length( VAL ) < 4    then return ERROR
      parse var VAL EAT 3 SIZE 5 VAL
      SIZE = c2d( reverse( SIZE ))
      if EAT <> ASCII         then return ERROR
      if length( VAL ) < SIZE then return ERROR
      call value THIS... || 0, N
      call value THIS... || N, left( VAL, SIZE )
      VAL = substr( VAL, SIZE + 1 )
   end N
   if VAL == ''               then return 0        /* really okay */
   if VAL == d2c( 0 )         then return 1        /* almost okay */
                              else return ERROR    /* extra bytes */

XEAT.PUTEA: procedure expose ( THIS... )
   call UTIL 'SysPutEA'          ;  MULTI = reverse( x2c( 'FFDF' ))
   ERROR = 13  ;  P = '.'        ;  ASCII = reverse( x2c( 'FFFD' ))
   LAST = value( THIS... || 0 )  ;  VAL  = ''   ;  N = 0
   if symbol(    THIS... || P ) =  'VAR'      then do
      N = value( THIS... || P )  ;  if N = '' then N = 0
   end                           /* treat undefined codepage as 0 */

   if datatype( LAST, 'w' ) = 0  then return ERROR
   if 10000 <=  LAST | LAST < 0  then return ERROR
   if datatype( N   , 'w' ) = 0  then return ERROR
   if 10000 <=  N    | N    < 0  then return ERROR

   if LAST > 1 | N <> 0 then do  /* use MULTI only if neccessary: */
      N    = reverse( x2c( d2x( N   , 4 )))
      SIZE = reverse( x2c( d2x( LAST, 4 )))
      VAL  = MULTI || N || SIZE  /* x2c(d2x(X, 2*L)) <> d2c(X, L) */
   end
   do N = 1 to LAST              /* WPS accects / keeps void, but */
      EAT  = THIS... || N        /* NOVALUE is definitely illegal */
      if symbol( EAT ) <> 'VAR'  then return ERROR
      EAT  = value( EAT )
      SIZE = reverse( x2c( d2x( length( EAT ), 4 )))
      VAL  = VAL || ASCII || SIZE || EAT
   end N                         /* empty VAL clears the EA .TYPE */
   return SysPutEA( arg( 1 ), arg( 2 ), VAL )

UTIL: procedure                  /* load necessary RexxUtil entry */
   if RxFuncQuery(  arg( 1 )) then
      if RxFuncAdd( arg( 1 ), 'RexxUtil', arg( 1 )) then
         exit TRAP( "can't add RexxUtil"  arg( 1 ))
   return 0

TRAP:                            /* select REXX exception handler */
   call trace 'O' ;  trace N           /* don't trace interactive */
   parse source TRAP                   /* source on separate line */
   TRAP = x2c( 0D ) || right( '+++', 10 ) TRAP || x2c( 0D0A )
   TRAP = TRAP || right( '+++', 10 )   /* = standard trace prefix */
   TRAP = TRAP strip( condition( 'c' ) 'trap:' condition( 'd' ))
   select
      when wordpos( condition( 'c' ), 'ERROR FAILURE' ) > 0 then do
         if condition( 'd' ) > ''      /* need an additional line */
            then TRAP = TRAP || x2c( 0D0A ) || right( '+++', 10 )
         TRAP = TRAP '(RC' rc || ')'   /* any system error codes  */
         if condition( 'c' ) = 'FAILURE' then rc = -3
      end
      when wordpos( condition( 'c' ), 'HALT SYNTAX'   ) > 0 then do
         if condition( 'c' ) = 'HALT' then rc = 4
         if condition( 'd' ) > '' & condition( 'd' ) <> rc then do
            if condition( 'd' ) <> errortext( rc ) then do
               TRAP = TRAP || x2c( 0D0A ) || right( '+++', 10 )
               TRAP = TRAP errortext( rc )
            end                        /* future condition( 'd' ) */
         end                           /* may use errortext( rc ) */
         else  TRAP = TRAP errortext( rc )
         rc = -rc                      /* rc < 0: REXX error code */
      end
      when condition( 'c' ) = 'NOVALUE'  then rc = -2 /* dubious  */
      when condition( 'c' ) = 'NOTREADY' then rc = -1 /* dubious  */
      otherwise                        /* force non-zero whole rc */
         if datatype( value( 'RC' ), 'W' ) = 0 then rc = 1
         if rc = 0                             then rc = 1
         if condition() = '' then TRAP = TRAP arg( 1 )
   end                                 /* direct: TRAP( message ) */

   TRAP = TRAP || x2c( 0D0A ) || format( sigl, 6 )
   signal on syntax name TRAP.SIGL     /* throw syntax error 3... */
   if 0 < sigl & sigl <= sourceline()  /* if no handle for source */
      then TRAP = TRAP '*-*' strip( sourceline( sigl ))
      else TRAP = TRAP '+++ (source line unavailable)'
TRAP.SIGL:                             /* ...catch syntax error 3 */
   if abbrev( right( TRAP, 2 + 6 ), x2c( 0D0A )) then do
      TRAP = TRAP '+++ (source line unreadable)'   ;  rc = -rc
   end
   select
      when 0 then do                   /* in pipes STDERR: output */
         parse version TRAP.REXX . .   /* REXX/Personal: \dev\con */
         signal on syntax name TRAP.FAIL
         if TRAP.REXX = 'REXXSAA'      /* fails if no more handle */
            then call lineout 'STDERR'  , TRAP
            else call lineout '\dev\con', TRAP
      end
      when 0 then do                   /* OS/2 PM: RxMessageBox() */
         signal on syntax name TRAP.FAIL
         call RxMessageBox ,           /* fails if not in PMREXX  */
            translate( TRAP, ' ', x2c( 0D )), , 'CANCEL', 'WARNING'
      end                              /* replace any CR by blank */
      otherwise   say TRAP ; trace ?L  /* interactive Label trace */
   end

   if condition() = 'SIGNAL' then signal TRAP.EXIT
TRAP.CALL:  return rc                  /* continue after CALL ON  */
TRAP.FAIL:  say TRAP ;  rc = 0 - rc    /* force TRAP error output */
TRAP.EXIT:  exit   rc                  /* exit for any SIGNAL ON  */
